// File:    t_dvm3_matrix.cc
// Author:  Terry Gaetz

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of dvm3.cvs
//
// dvm3.cvs is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// dvm3.cvs is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <dvm3/dvm3_matrix.h>

#include <cstdlib>
#include <iostream>
#include <testlib/testlib.h>

using namespace std;

int dvm3_Matrix_test01()    /* test construct from 3 dvm3_Vector's */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  dvm3_Vector c( 2.0, 4.0, 8.0 );
  dvm3_Matrix m1( a, b, c );
  return 
    (m1[0][0] == 1.0) && (m1[0][1] == 2.0) && (m1[0][2] == 3.0) &&
    (m1[1][0] == 4.0) && (m1[1][1] == 5.0) && (m1[1][2] == 6.0) &&
    (m1[2][0] == 2.0) && (m1[2][1] == 4.0) && (m1[2][2] == 8.0);
}

static
int dvm3_Matrix_test02()    /* test construct from 3 double[]'s */
{
  double a[] = { 1.0, 2.0, 3.0 };
  double b[] = { 4.0, 5.0, 6.0 };
  double c[] = { 2.0, 4.0, 8.0 };
  dvm3_Matrix m1( a, b, c );
  return 
    (m1[0][0] == 1.0) && (m1[0][1] == 2.0) && (m1[0][2] == 3.0) &&
    (m1[1][0] == 4.0) && (m1[1][1] == 5.0) && (m1[1][2] == 6.0) &&
    (m1[2][0] == 2.0) && (m1[2][1] == 4.0) && (m1[2][2] == 8.0);
}

static
int dvm3_Matrix_test03()    /* test copy constructor */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  dvm3_Vector c( 2.0, 4.0, 8.0 );
  dvm3_Matrix m1( a, b, c );
  dvm3_Matrix m2( m1 );
  return
    (m1[0][0] == 1.0) && (m1[0][1] == 2.0) && (m1[0][2] == 3.0) &&
    (m1[1][0] == 4.0) && (m1[1][1] == 5.0) && (m1[1][2] == 6.0) &&
    (m1[2][0] == 2.0) && (m1[2][1] == 4.0) && (m1[2][2] == 8.0) &&
    (m2[0][0] == 1.0) && (m2[0][1] == 2.0) && (m2[0][2] == 3.0) &&
    (m2[1][0] == 4.0) && (m2[1][1] == 5.0) && (m2[1][2] == 6.0) &&
    (m2[2][0] == 2.0) && (m2[2][1] == 4.0) && (m2[2][2] == 8.0);
}

static
int dvm3_Matrix_test04()    /* test init_by_row: 3 dvm3_Vector's */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  dvm3_Vector c( 2.0, 4.0, 8.0 );
  dvm3_Matrix m1;
  m1.init_by_row( a, b, c );
  return 
    (m1[0][0] == 1.0) && (m1[0][1] == 2.0) && (m1[0][2] == 3.0) &&
    (m1[1][0] == 4.0) && (m1[1][1] == 5.0) && (m1[1][2] == 6.0) &&
    (m1[2][0] == 2.0) && (m1[2][1] == 4.0) && (m1[2][2] == 8.0);
}

static
int dvm3_Matrix_test05()    /* test init_by_row: 3 double[]'s */
{
  double a[] = { 1.0, 2.0, 3.0 };
  double b[] = { 4.0, 5.0, 6.0 };
  double c[] = { 2.0, 4.0, 8.0 };
  dvm3_Matrix m1;
  m1.init_by_row( a, b, c );
  return 
    (m1[0][0] == 1.0) && (m1[0][1] == 2.0) && (m1[0][2] == 3.0) &&
    (m1[1][0] == 4.0) && (m1[1][1] == 5.0) && (m1[1][2] == 6.0) &&
    (m1[2][0] == 2.0) && (m1[2][1] == 4.0) && (m1[2][2] == 8.0);
}

static
int dvm3_Matrix_test06()    /* test init_by_col: 3 dvm3_Vector's */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  dvm3_Vector c( 2.0, 4.0, 8.0 );
  dvm3_Matrix m1;
  m1.init_by_col( a, b, c );
  return 
    (m1[0][0] == 1.0) && (m1[1][0] == 2.0) && (m1[2][0] == 3.0) &&
    (m1[0][1] == 4.0) && (m1[1][1] == 5.0) && (m1[2][1] == 6.0) &&
    (m1[0][2] == 2.0) && (m1[1][2] == 4.0) && (m1[2][2] == 8.0);
}

static
int dvm3_Matrix_test07()    /* test init_by_col: 3 double[]'s */
{
  double a[] = { 1.0, 2.0, 3.0 };
  double b[] = { 4.0, 5.0, 6.0 };
  double c[] = { 2.0, 4.0, 8.0 };
  dvm3_Matrix m1;
  m1.init_by_col( a, b, c );
  return 
    (m1[0][0] == 1.0) && (m1[1][0] == 2.0) && (m1[2][0] == 3.0) &&
    (m1[0][1] == 4.0) && (m1[1][1] == 5.0) && (m1[2][1] == 6.0) &&
    (m1[0][2] == 2.0) && (m1[1][2] == 4.0) && (m1[2][2] == 8.0);
}

static
int dvm3_Matrix_test08()    /* test copy-assignment */
{
  double a[] = { 1.0, 2.0, 3.0 };
  double b[] = { 4.0, 5.0, 6.0 };
  double c[] = { 2.0, 4.0, 8.0 };
  double d[] = { -1.0, -2.0, -3.0 };
  double e[] = { -4.0, -5.0, -6.0 };
  double f[] = { -2.0, -4.0, -8.0 };
  dvm3_Matrix m1( a, b, c );
  dvm3_Matrix m2;
  m2 = m1;
  m1.init_by_row( d, e, f );
  return 
    (m1[0][0] == -1.0) && (m1[0][1] == -2.0) && (m1[0][2] == -3.0) &&
    (m1[1][0] == -4.0) && (m1[1][1] == -5.0) && (m1[1][2] == -6.0) &&
    (m1[2][0] == -2.0) && (m1[2][1] == -4.0) && (m1[2][2] == -8.0) &&
    (m2[0][0] ==  1.0) && (m2[0][1] ==  2.0) && (m2[0][2] ==  3.0) &&
    (m2[1][0] ==  4.0) && (m2[1][1] ==  5.0) && (m2[1][2] ==  6.0) &&
    (m2[2][0] ==  2.0) && (m2[2][1] ==  4.0) && (m2[2][2] ==  8.0);
}

static
int dvm3_Matrix_test09()    /* test operator=(double) */
{
  dvm3_Matrix m1;
  m1 = 1.7;
  return
    (m1[0][0] == 1.7) && (m1[0][1] == 1.7) && (m1[0][2] == 1.7) &&
    (m1[1][0] == 1.7) && (m1[1][1] == 1.7) && (m1[1][2] == 1.7) &&
    (m1[2][0] == 1.7) && (m1[2][1] == 1.7) && (m1[2][2] == 1.7);
}

static
int dvm3_Matrix_test10()    /* test a {+,-,*,/}= c */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  dvm3_Vector c( 2.0, 4.0, 8.0 );

  dvm3_Matrix m1( a, b, c );
  dvm3_Matrix m3;  m3.init_by_row( a, b, c );
  m3 += m1;
  m3 -= m1;
  m3 *= m1;
  m3 /= m1;
  return !fuzzy_compare(m3[0][0], 1.0) &&
         !fuzzy_compare(m3[0][1], 2.0) &&
         !fuzzy_compare(m3[0][2], 3.0) &&
         !fuzzy_compare(m3[1][0], 4.0) &&
         !fuzzy_compare(m3[1][1], 5.0) &&
         !fuzzy_compare(m3[1][2], 6.0) &&
         !fuzzy_compare(m3[2][0], 2.0) &&
         !fuzzy_compare(m3[2][1], 4.0) &&
         !fuzzy_compare(m3[2][2], 8.0);
}

static
int dvm3_Matrix_test11()    /* test a {+,-,*,/}= double */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  dvm3_Vector c( 2.0, 4.0, 8.0 );

  dvm3_Matrix m1( a, b, c );
  dvm3_Matrix m3;  m3.init_by_row( a, b, c );
  m3 += 2.0;
  m3 -= 2.0;
  m3 *= 2.0;
  m3 /= 2.0;
  return !fuzzy_compare(m3[0][0], 1.0) &&
         !fuzzy_compare(m3[0][1], 2.0) &&
         !fuzzy_compare(m3[0][2], 3.0) &&
         !fuzzy_compare(m3[1][0], 4.0) &&
         !fuzzy_compare(m3[1][1], 5.0) &&
         !fuzzy_compare(m3[1][2], 6.0) &&
         !fuzzy_compare(m3[2][0], 2.0) &&
         !fuzzy_compare(m3[2][1], 4.0) &&
         !fuzzy_compare(m3[2][2], 8.0);
}

static
int dvm3_Matrix_test12()    /* test operator+() */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  dvm3_Vector c( 2.0, 4.0, 8.0 );

  dvm3_Matrix m1( a, b, c );
  dvm3_Matrix m2;
  m2 = +m1;
  return 
    (m1[0][0] ==  1.0) && (m1[0][1] ==  2.0) && (m1[0][2] ==  3.0) &&
    (m1[1][0] ==  4.0) && (m1[1][1] ==  5.0) && (m1[1][2] ==  6.0) &&
    (m1[2][0] ==  2.0) && (m1[2][1] ==  4.0) && (m1[2][2] ==  8.0) &&
    (m2[0][0] ==  1.0) && (m2[0][1] ==  2.0) && (m2[0][2] ==  3.0) &&
    (m2[1][0] ==  4.0) && (m2[1][1] ==  5.0) && (m2[1][2] ==  6.0) &&
    (m2[2][0] ==  2.0) && (m2[2][1] ==  4.0) && (m2[2][2] ==  8.0);
}

static
int dvm3_Matrix_test13()    /* test operator-() */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  dvm3_Vector c( 2.0, 4.0, 8.0 );

  dvm3_Matrix m1( a, b, c );
  dvm3_Matrix m2;
  m2 = -m1;
  return 
    (m1[0][0] == -1.0) && (m1[0][1] == -2.0) && (m1[0][2] == -3.0) &&
    (m1[1][0] == -4.0) && (m1[1][1] == -5.0) && (m1[1][2] == -6.0) &&
    (m1[2][0] == -2.0) && (m1[2][1] == -4.0) && (m1[2][2] == -8.0) &&
    (m2[0][0] == -1.0) && (m2[0][1] == -2.0) && (m2[0][2] == -3.0) &&
    (m2[1][0] == -4.0) && (m2[1][1] == -5.0) && (m2[1][2] == -6.0) &&
    (m2[2][0] == -2.0) && (m2[2][1] == -4.0) && (m2[2][2] == -8.0);
}

static
int dvm3_Matrix_test14()    /* test operator{+,-,*,/}(cm,cm) */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  dvm3_Vector c( 2.0, 4.0, 8.0 );
  dvm3_Vector d( 2.0, 1.0, 4.0 );
  dvm3_Vector e( 6.0, 4.0, 5.0 );
  dvm3_Vector f( 3.0, 2.0, 1.0 );
  dvm3_Matrix ma( a, b, c );
  dvm3_Matrix mb;
  dvm3_Matrix mc( d, e, f );
  mb = ma + ma;
  mb = mb - ma;
  mb = ma * mc;
  mb = mb / mc;
  return !fuzzy_compare(mb[0][0], 1.0) &&
         !fuzzy_compare(mb[0][1], 2.0) &&
         !fuzzy_compare(mb[0][2], 3.0) &&
         !fuzzy_compare(mb[1][0], 4.0) &&
         !fuzzy_compare(mb[1][1], 5.0) &&
         !fuzzy_compare(mb[1][2], 6.0) &&
         !fuzzy_compare(mb[2][0], 2.0) &&
         !fuzzy_compare(mb[2][1], 4.0) &&
         !fuzzy_compare(mb[2][2], 8.0);
}

static
int dvm3_Matrix_test15()    /* test inject_row (dvm3_Vector) */
{
  dvm3_Vector a( -1.0, -2.0, -3.0 );
  dvm3_Vector b( -4.0, -5.0, -6.0 );
  dvm3_Vector c( -2.0, -4.0, -8.0 );
  dvm3_Matrix m1;  m1.init_by_row( a, b, c );
  dvm3_Vector x(  1.0,  2.0,  3.0 );
  m1.inject_row( 2, x );
  return
    (m1[0][0] == -1.0) && (m1[0][1] == -2.0) && (m1[0][2] == -3.0) &&
    (m1[1][0] == -4.0) && (m1[1][1] == -5.0) && (m1[1][2] == -6.0) &&
    (m1[2][0] ==  1.0) && (m1[2][1] ==  2.0) && (m1[2][2] ==  3.0);
}

static
int dvm3_Matrix_test16()    /* test inject_row (double[]) */
{
  double a[] = { -1.0, -2.0, -3.0 };
  double b[] = { -4.0, -5.0, -6.0 };
  double c[] = { -2.0, -4.0, -8.0 };
  dvm3_Matrix m1;  m1.init_by_row( a, b, c );
  double x[] = {  1.0,  2.0,  3.0 };
  m1.inject_row( 1, x );
  return
    (m1[0][0] == -1.0) && (m1[0][1] == -2.0) && (m1[0][2] == -3.0) &&
    (m1[1][0] ==  1.0) && (m1[1][1] ==  2.0) && (m1[1][2] ==  3.0) &&
    (m1[2][0] == -2.0) && (m1[2][1] == -4.0) && (m1[2][2] == -8.0);
}

static
int dvm3_Matrix_test17()    /* test inject_col (dvm3_Vector) */
{
  dvm3_Vector a( -1.0, -2.0, -3.0 );
  dvm3_Vector b( -4.0, -5.0, -6.0 );
  dvm3_Vector c( -2.0, -4.0, -8.0 );
  dvm3_Matrix m1;  m1.init_by_row( a, b, c );
  dvm3_Vector x(  1.0,  2.0,  3.0 );
  m1.inject_col( 2, x );
  return
    (m1[0][0] == -1.0) && (m1[0][1] == -2.0) && (m1[0][2] ==  1.0) &&
    (m1[1][0] == -4.0) && (m1[1][1] == -5.0) && (m1[1][2] ==  2.0) &&
    (m1[2][0] == -2.0) && (m1[2][1] == -4.0) && (m1[2][2] ==  3.0);
}

static
int dvm3_Matrix_test18()    /* test inject_col (double[]) */
{
  double a[] = { -1.0, -2.0, -3.0 };
  double b[] = { -4.0, -5.0, -6.0 };
  double c[] = { -2.0, -4.0, -8.0 };
  dvm3_Matrix m1;  m1.init_by_row( a, b, c );
  double x[] = {  1.0,  2.0,  3.0 };
  m1.inject_col( 1, x );
  return
    (m1[0][0] == -1.0) && (m1[0][1] ==  1.0) && (m1[0][2] == -3.0) &&
    (m1[1][0] == -4.0) && (m1[1][1] ==  2.0) && (m1[1][2] == -6.0) &&
    (m1[2][0] == -2.0) && (m1[2][1] ==  3.0) && (m1[2][2] == -8.0);
}

static
int dvm3_Matrix_test19()    /* test extract_row (dvm3_Vector) */
{
  dvm3_Vector a( -1.0, -2.0, -3.0 );
  dvm3_Vector b( -4.0, -5.0, -6.0 );
  dvm3_Vector c( -2.0, -4.0, -8.0 );
  dvm3_Matrix m1;  m1.init_by_row( a, b, c );
  dvm3_Vector x(  1.0,  2.0,  3.0 );
  m1.extract_row( 2, x );
  return
    (x[0] == -2.0) && (x[1] == -4.0) && (x[2] == -8.0);
}

static
int dvm3_Matrix_test20()    /* test extract_row (double[]) */
{
  double a[] = { -1.0, -2.0, -3.0 };
  double b[] = { -4.0, -5.0, -6.0 };
  double c[] = { -2.0, -4.0, -8.0 };
  dvm3_Matrix m1;  m1.init_by_row( a, b, c );
  double x[] = {  1.0,  2.0,  3.0 };
  m1.extract_row( 1, x );
  return
    (x[0] == -4.0) && (x[1] == -5.0) && (x[2] == -6.0);
}

static
int dvm3_Matrix_test21()    /* test extract_col (dvm3_Vector) */
{
  dvm3_Vector a( -1.0, -2.0, -3.0 );
  dvm3_Vector b( -4.0, -5.0, -6.0 );
  dvm3_Vector c( -2.0, -4.0, -8.0 );
  dvm3_Matrix m1;  m1.init_by_col( a, b, c );
  dvm3_Vector x(  1.0,  2.0,  3.0 );
  m1.extract_col( 2, x );
  return
    (x[0] == -2.0) && (x[1] == -4.0) && (x[2] == -8.0);
}

static
int dvm3_Matrix_test22()    /* test extract_col (double[]) */
{
  double a[] = { -1.0, -2.0, -3.0 };
  double b[] = { -4.0, -5.0, -6.0 };
  double c[] = { -2.0, -4.0, -8.0 };
  dvm3_Matrix m1;  m1.init_by_col( a, b, c );
  double x[] = {  1.0,  2.0,  3.0 };
  m1.extract_col( 1, x );
  return
    (x[0] == -4.0) && (x[1] == -5.0) && (x[2] == -6.0);
}

static
int dvm3_Matrix_test23()    /* test dyad_product:  dvm3_Vector */
{
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  dvm3_Vector c( 2.0, 4.0, 8.0 );
  dvm3_Matrix m1;
  m1.dyad_product(b, c);
  return
     (m1[0][0] ==  8.0) && (m1[0][1] == 16.0) && (m1[0][2] == 32.0) &&
     (m1[1][0] == 10.0) && (m1[1][1] == 20.0) && (m1[1][2] == 40.0) &&
     (m1[2][0] == 12.0) && (m1[2][1] == 24.0) && (m1[2][2] == 48.0);
}

static
int dvm3_Matrix_test24()    /* test dyad_product:  double[] */
{
  double b[] = { 4.0, 5.0, 6.0 };
  double c[] = { 2.0, 4.0, 8.0 };
  dvm3_Matrix m1;
  m1.dyad_product(b, c);
  return
     (m1[0][0] ==  8.0) && (m1[0][1] == 16.0) && (m1[0][2] == 32.0) &&
     (m1[1][0] == 10.0) && (m1[1][1] == 20.0) && (m1[1][2] == 40.0) &&
     (m1[2][0] == 12.0) && (m1[2][1] == 24.0) && (m1[2][2] == 48.0);
}

static
int dvm3_Matrix_test25()    /* test m3.lincomb(2,m1,3,m2) */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  dvm3_Vector c( 2.0, 4.0, 8.0 );
  dvm3_Vector d( 2.0, 1.0, 4.0 );
  dvm3_Vector e( 6.0, 4.0, 5.0 );
  dvm3_Vector f( 3.0, 2.0, 1.0 );
  dvm3_Matrix m1( a, b, c );
  dvm3_Matrix m2( d, e, f );
  dvm3_Matrix m3;
  m3.lincomb( 2.0, m1, 3.0, m2 );
  return
     (m3[0][0] ==  8.0) && (m3[0][1] ==  7.0) && (m3[0][2] == 18.0) &&
     (m3[1][0] == 26.0) && (m3[1][1] == 22.0) && (m3[1][2] == 27.0) &&
     (m3[2][0] == 13.0) && (m3[2][1] == 14.0) && (m3[2][2] == 19.0);
}

static
int dvm3_Matrix_test26()    /* test lincomb(m3,2,m1,3,m2) */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  dvm3_Vector c( 2.0, 4.0, 8.0 );
  dvm3_Vector d( 2.0, 1.0, 4.0 );
  dvm3_Vector e( 6.0, 4.0, 5.0 );
  dvm3_Vector f( 3.0, 2.0, 1.0 );
  dvm3_Matrix m1( a, b, c );
  dvm3_Matrix m2( d, e, f );
  dvm3_Matrix m3;
  lincomb( m3, 2.0, m1, 3.0, m2 );
  return
     (m3[0][0] ==  8.0) && (m3[0][1] ==  7.0) && (m3[0][2] == 18.0) &&
     (m3[1][0] == 26.0) && (m3[1][1] == 22.0) && (m3[1][2] == 27.0) &&
     (m3[2][0] == 13.0) && (m3[2][1] == 14.0) && (m3[2][2] == 19.0);
}

static
int dvm3_Matrix_test27()    /* test m1.mvmult(v2, cv1)  dvm3_Vector */
{
  dvm3_Vector d( 2.0, 1.0, 4.0 );
  dvm3_Vector e( 6.0, 4.0, 5.0 );
  dvm3_Vector f( 3.0, 2.0, 1.0 );
  dvm3_Matrix m1( d, e, f );
  dvm3_Vector v1( 2.0, 3.0, 4.0 );
  dvm3_Vector v2;
  m1.mvmult( v2, v1 );
  return
     (v1[0] ==  2.0) && (v1[1] ==  3.0) && (v1[2] ==  4.0) &&
     (v2[0] == 23.0) && (v2[1] == 44.0) && (v2[2] == 16.0);
}

static
int dvm3_Matrix_test28()    /* test mvmult(v2, cm, cv1)  double[] */
{
  dvm3_Vector d( 2.0, 1.0, 4.0 );
  dvm3_Vector e( 6.0, 4.0, 5.0 );
  dvm3_Vector f( 3.0, 2.0, 1.0 );
  dvm3_Matrix m1( d, e, f );
  double v1[] = { 2.0, 3.0, 4.0 };
  double v2[3];
  mvmult( v2, m1, v1 );
  return
     (v1[0] ==  2.0) && (v1[1] ==  3.0) && (v1[2] ==  4.0) &&
     (v2[0] == 23.0) && (v2[1] == 44.0) && (v2[2] == 16.0);
}

static
int dvm3_Matrix_test29()    /* test mtvmult(v2, cm, cv1)  dvm3_Vector */
{
  dvm3_Vector d( 2.0, 1.0, 4.0 );
  dvm3_Vector e( 6.0, 4.0, 5.0 );
  dvm3_Vector f( 3.0, 2.0, 1.0 );
  dvm3_Matrix m1( d, e, f );
  dvm3_Vector v1( 2.0, 3.0, 4.0 );
  dvm3_Vector v2;
  mtvmult( v2, m1, v1 );
  return
     (v1[0] ==  2.0) && (v1[1] ==  3.0) && (v1[2] ==  4.0) &&
     (v2[0] == 34.0) && (v2[1] == 22.0) && (v2[2] == 27.0);
}

static
int dvm3_Matrix_test30()    /* test mvmult(v2, cm, cv1)  double[] */
{
  dvm3_Vector d( 2.0, 1.0, 4.0 );
  dvm3_Vector e( 6.0, 4.0, 5.0 );
  dvm3_Vector f( 3.0, 2.0, 1.0 );
  dvm3_Matrix m1( d, e, f );
  double v1[] = { 2.0, 3.0, 4.0 };
  double v2[3];
  mtvmult( v2, m1, v1 );
  return
     (v1[0] ==  2.0) && (v1[1] ==  3.0) && (v1[2] ==  4.0) &&
     (v2[0] == 34.0) && (v2[1] == 22.0) && (v2[2] == 27.0);
}

static
int dvm3_Matrix_test31()    /* test mmult(m3, m1, m2) */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  dvm3_Vector c( 2.0, 4.0, 8.0 );
  dvm3_Vector d( 2.0, 1.0, 4.0 );
  dvm3_Vector e( 6.0, 4.0, 5.0 );
  dvm3_Vector f( 3.0, 2.0, 1.0 );
  dvm3_Matrix m1( a, b, c );
  dvm3_Matrix m2( d, e, f );
  dvm3_Matrix m3;
  mmult( m3, m1, m2 );
  return
     (m3[0][0] == 23.0) && (m3[0][1] == 15.0) && (m3[0][2] == 17.0) &&
     (m3[1][0] == 56.0) && (m3[1][1] == 36.0) && (m3[1][2] == 47.0) &&
     (m3[2][0] == 52.0) && (m3[2][1] == 34.0) && (m3[2][2] == 36.0);
}

int
main( int argc, char** argv )
{
  int ch;
  int printit = 0;
  while ( --argc > 0 && (*++argv)[0] == '-' )
  {
    while ( (ch = *++argv[0]) )
    {
      switch( ch )
      {
      case 'p':                   // print out diagnostics as we go along
        printit = 1;
        break;
      default:
        cerr << "Usage: " << argv[0] << " [-p]\n";
        return EXIT_FAILURE;
      }
    }
  }

  START("dvm3 package:  dvm3_Matrix");

  TEST( "construct from dvm3_Vector's",     dvm3_Matrix_test01(), 1 );
  TEST( "construct from double[]'s",        dvm3_Matrix_test02(), 1 );

  TEST( "copy constructor",                 dvm3_Matrix_test03(), 1 );

  TEST( "init_by_row: dvm3_Vector's",       dvm3_Matrix_test04(), 1 );
  TEST( "init_by_row: double[]'s",          dvm3_Matrix_test05(), 1 );

  TEST( "init_by_col: dvm3_Vector's",       dvm3_Matrix_test06(), 1 );
  TEST( "init_by_col: double[]'s",          dvm3_Matrix_test07(), 1 );

  TEST( "operator=(dvm3_Matrix)",           dvm3_Matrix_test08(), 1 );
  TEST( "operator=(double)",                dvm3_Matrix_test09(), 1 );

  TEST( "a {+,-,*,/}= c",                   dvm3_Matrix_test10(), 1 );
  TEST( "a {+,-,*,/}= double",              dvm3_Matrix_test11(), 1 );
  TEST( "operator+()",                      dvm3_Matrix_test12(), 1 );
  TEST( "operator-()",                      dvm3_Matrix_test13(), 1 );
  TEST( "operator{+,-,*,/}(cm,cm)",         dvm3_Matrix_test14(), 1 );

  TEST( "inject_row: dvm3_Vector's",        dvm3_Matrix_test15(), 1 );
  TEST( "inject_row: double[]'s",           dvm3_Matrix_test16(), 1 );

  TEST( "inject_col: dvm3_Vector's",        dvm3_Matrix_test17(), 1 );
  TEST( "inject_col: double[]'s",           dvm3_Matrix_test18(), 1 );

  TEST( "extract_row: dvm3_Vector's",       dvm3_Matrix_test19(), 1 );
  TEST( "extract_row: double[]'s",          dvm3_Matrix_test20(), 1 );

  TEST( "extract_col: dvm3_Vector's",       dvm3_Matrix_test21(), 1 );
  TEST( "extract_col: double[]'s",          dvm3_Matrix_test22(), 1 );

  TEST( "dyad_product: dvm3_Vector's",      dvm3_Matrix_test23(), 1 );
  TEST( "dyad_product: double[]'s",         dvm3_Matrix_test24(), 1 );

  TEST( "m3.lincomb(2,m1,3,m2)",            dvm3_Matrix_test25(), 1 );
  TEST( "lincomb(m3,2,m1,3,m2)",            dvm3_Matrix_test26(), 1 );

  TEST( "mvmult(v2, cm, v): dvm3_Vector's",  dvm3_Matrix_test27(), 1 );
  TEST( "mvmult(v2, cm, v): double[]'s",     dvm3_Matrix_test28(), 1 );

  TEST( "mtvmult(v2, cm, v): dvm3_Vector's", dvm3_Matrix_test29(), 1 );
  TEST( "mtvmult(v2, cm, v): double[]'s",    dvm3_Matrix_test30(), 1 );

  TEST( "mmult(m3, m1, m2)",                 dvm3_Matrix_test31(), 1 );

  SUMMARY();

  return FAILED() ? 1 : 0;
}
