// File:    t_dvm3_vector.cc
// Author:  Terry Gaetz

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of dvm3.cvs
//
// dvm3.cvs is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// dvm3.cvs is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <dvm3/dvm3_vector.h>

#include <cstdlib>
#include <iostream>
#include <testlib/testlib.h>

using namespace std;

static
int dvm3_Vector_test01()    /* test dvm3_Vector(d,d,d) */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  return (a(0) == 1.0) && (a(1) == 2.0) && (a(2) == 3.0);
}

static
int dvm3_Vector_test02()    /* test v.copy_from_cvec(cv) */
{
  double      bcv[3]  = { 4.0, 5.0, 6.0 };
  dvm3_Vector b;
  b.copy_from_cvec( bcv );
  return (b(0) == 4.0) && (b(1) == 5.0) && (b(2) == 6.0);
}

static
int dvm3_Vector_test03()    /* test v.copy_to_cvec(a) */
{
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  double      a[3];
  b.copy_to_cvec( a );
  return (a[0] == 4.0) && (a[1] == 5.0) && (a[2] == 6.0);
}

static
int dvm3_Vector_test04()    /* test b = 3.0 */
{
  dvm3_Vector b;
  b = 3;
  return (b(0) == 3.0) && (b(1) == 3.0) && (b(2) == 3.0);
}

static
int dvm3_Vector_test05()    /* test b = a */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  dvm3_Vector b;
  b = a;
  return (b(0) == 1.0) && (b(1) == 2.0) && (b(2) == 3.0);
}

static
int dvm3_Vector_test06()    /* test a {+,-,*,/}= c */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  dvm3_Vector c( 2.0, 4.0, 8.0 );
  a += c;
  a -= c;
  a *= c;
  a /= c;
  return !fuzzy_compare(a(0), 1.0) &&
         !fuzzy_compare(a(1), 2.0) &&
         !fuzzy_compare(a(2), 3.0);
}

static
int dvm3_Vector_test07()    /* test operator{+,-,*,/}(cv,cv) */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  dvm3_Vector b;
  dvm3_Vector c( 2.0, 4.0, 8.0 );
  b = a + a;
  b = b - a;
  b = a * c;
  b = b / c;
  return !fuzzy_compare(b(0), 1.0) &&
         !fuzzy_compare(b(1), 2.0) &&
         !fuzzy_compare(b(2), 3.0);
}

static
int dvm3_Vector_test08()    /* test operator+() */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  dvm3_Vector b;
  b = +a;
  return !fuzzy_compare(a(0), 1.0) &&
         !fuzzy_compare(a(1), 2.0) &&
         !fuzzy_compare(a(2), 3.0) &&
         !fuzzy_compare(b(0), 1.0) &&
         !fuzzy_compare(b(1), 2.0) &&
         !fuzzy_compare(b(2), 3.0);
}

static
int dvm3_Vector_test09()    /* test operator-() */
{
  dvm3_Vector a( 1.0, 2.0, 3.0 );
  dvm3_Vector b;
  b = -a;
  return !fuzzy_compare(a(0), -1.0) &&
         !fuzzy_compare(a(1), -2.0) &&
         !fuzzy_compare(a(2), -3.0) &&
         !fuzzy_compare(b(0), -1.0) &&
         !fuzzy_compare(b(1), -2.0) &&
         !fuzzy_compare(b(2), -3.0);
}

static
int dvm3_Vector_test10()    /* test c.lincomb(3,a,2,b) */
{
  dvm3_Vector a( 2.0, 3.0, 4.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  dvm3_Vector c;
  c.lincomb(3.0, a, 2.0, b);
  return !fuzzy_compare(c(0), 14.0) &&
         !fuzzy_compare(c(1), 19.0) &&
         !fuzzy_compare(c(2), 24.0);
}

static
int dvm3_Vector_test11()    /* test lincomb(c,3,a,2,b) */
{
  dvm3_Vector a( 2.0, 3.0, 4.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  dvm3_Vector c;
  lincomb(c, 3.0, a, 2.0, b);
  return !fuzzy_compare(c(0), 14.0) &&
         !fuzzy_compare(c(1), 19.0) &&
         !fuzzy_compare(c(2), 24.0);
}

static
int dvm3_Vector_test12()    /* test a.dot(b) */
{
  dvm3_Vector a( 2.0, 3.0, 4.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  return !fuzzy_compare(a.dot(b), 47.0);
}

static
int dvm3_Vector_test13()    /* test dot(a,b) */
{
  dvm3_Vector a( 2.0, 3.0, 4.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  return !fuzzy_compare(dot(a,b), 47.0);
}

static
int dvm3_Vector_test14()    /* test c.cross(a,b) */
{
  dvm3_Vector a( 2.0, 3.0, 4.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  dvm3_Vector c;
  c.cross(a,b);
  return !fuzzy_compare(c(0), -2.0) &&
         !fuzzy_compare(c(1),  4.0) &&
         !fuzzy_compare(c(2), -2.0);
}

static
int dvm3_Vector_test15()    /* test cross(c,a,b) */
{
  dvm3_Vector a( 2.0, 3.0, 4.0 );
  dvm3_Vector b( 4.0, 5.0, 6.0 );
  dvm3_Vector c;
  cross(c,a,b);
  return !fuzzy_compare(c(0), -2.0) &&
         !fuzzy_compare(c(1),  4.0) &&
         !fuzzy_compare(c(2), -2.0);
}

static
int dvm3_Vector_test16()    /* test a.is_unit_vector() */
{
  dvm3_Vector a( cos(0.1), sin(0.1), 0.0      );
  dvm3_Vector b( cos(0.1), 0.0,      sin(0.1) );
  dvm3_Vector c( 0.0,     -sin(0.2), cos(0.2) );
  return a.is_unit_vector() && b.is_unit_vector() && c.is_unit_vector();
}

static
int dvm3_Vector_test17()    /* test a.is_orthogonal_to(b) */
{
  dvm3_Vector a(  cos(0.1),  sin(0.1), 0.0      );
  dvm3_Vector b( -sin(0.1),  cos(0.1), 0.0      );
  return a.is_orthogonal_to(b);
}

static
int dvm3_Vector_test18()    /* test a.unitize() */
{
  dvm3_Vector a( 4.0, 5.0, 6.0 );
  double rl = sqrt( dot(a, a) );
  double rv = a.unitize();
  return !fuzzy_compare(a(0), 4.0/rl) &&
         !fuzzy_compare(a(1), 5.0/rl) &&
         !fuzzy_compare(a(2), 6.0/rl) &&
         !fuzzy_compare(rv, rl) &&
         a.is_unit_vector();
}

static
int dvm3_Vector_test19()    /* test unitize(v, cv) */
{
  dvm3_Vector a( 4.0, 5.0, 6.0 );
  dvm3_Vector b;
  double rl = sqrt( a.dot(a) );
  double rv = unitize( b, a );
  return (a(0) == 4.0) && (a(1) == 5.0) && (a(2) == 6.0) &&
         !fuzzy_compare(b(0), 4.0/rl) &&
         !fuzzy_compare(b(1), 5.0/rl) &&
         !fuzzy_compare(b(2), 6.0/rl) &&
         !fuzzy_compare(rv, rl) &&
         b.is_unit_vector();
}

int
main( int argc, char** argv )
{
  int ch;
  int printit = 0;
  while ( --argc > 0 && (*++argv)[0] == '-' )
  {
    while ( (ch = *++argv[0]) )
    {
      switch( ch )
      {
      case 'p':                   // print out diagnostics as we go along
        printit = 1;
        break;
      default:
        cerr << "Usage: " << argv[0] << " [-p]\n";
        return EXIT_FAILURE;
      }
    }
  }

  START("dvm3 package:  dvm3_Vector");
  TEST( "dvm3_Vector(d,d,d)",         dvm3_Vector_test01(), 1 );
  TEST( "v.copy_from_cvec(a)",        dvm3_Vector_test02(), 1 );
  TEST( "v.copy_to_cvec(a)",          dvm3_Vector_test03(), 1 );
  TEST( "b = 3.0",                    dvm3_Vector_test04(), 1 );
  TEST( "b = a",                      dvm3_Vector_test05(), 1 );
  TEST( "a {+,-,*,/}= c",             dvm3_Vector_test06(), 1 );
  TEST( "operator{+,-,*,/}(cv,cv)",   dvm3_Vector_test07(), 1 );
  TEST( "operator+()",                dvm3_Vector_test08(), 1 );
  TEST( "operator-()",                dvm3_Vector_test09(), 1 );
  TEST( "c.lincomb(3,a,2,b)",         dvm3_Vector_test10(), 1 );
  TEST( "lincomb(c,3,a,2,b)",         dvm3_Vector_test11(), 1 );
  TEST( "a.dot(b)",                   dvm3_Vector_test12(), 1 );
  TEST( "dot(a,b)",                   dvm3_Vector_test13(), 1 );
  TEST( "c.cross(a,b)",               dvm3_Vector_test14(), 1 );
  TEST( "cross(c,a,b)",               dvm3_Vector_test15(), 1 );
  TEST( "a.is_unit_vector()",         dvm3_Vector_test16(), 1 );
  TEST( "a.is_orthogonal_to(b)",      dvm3_Vector_test17(), 1 );
  TEST( "a.unitize()",                dvm3_Vector_test18(), 1 );
  TEST( "unitize(v,cv)",              dvm3_Vector_test19(), 1 );
  SUMMARY();

  return FAILED() ? 1 : 0;
}

